'use strict';

exports.__esModule = true;
exports['default'] = combineReducersWithReduxables;

var _redux = require('redux');

var _Reduxable = require('./Reduxable');

var _Reduxable2 = _interopRequireDefault(_Reduxable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

/**
 *
 * The only difference with Redux's `combineReducers` is that it also recognizes
 * the Reduxable instances getting its reducers.
 *
 * @returns {Function} A reducer function that invokes every reducer inside the
 * passed object, and builds a state object with the same shape.
 */

function combineReducersWithReduxables(reducers) {
  // TODO: check for Reduxable inhetirance properly
  //       Don't know why `reducer.constructor.prototype instanceof Reduxable` is not working
  var newReducers = {};
  Object.keys(reducers).forEach(function (key) {
    var reducer = reducers[key];
    if (reducer.setScope) {
      reducer.setScope(key);
    }

    if (reducer.getReducer) {
      newReducers[key] = reducer.getReducer();
    } else {
      newReducers[key] = reducer;
    }
  });

  var reducer = (0, _redux.combineReducers)(newReducers);
  reducer.setScope = function (scope) {
    Object.keys(reducers).forEach(function (key) {
      var reducer = reducers[key];
      if (reducer.setScope) {
        reducer.setScope(scope + '.' + key);
      }
    });
  };
  return reducer;
}